//
// SkinnedWndCompanionObj.h
//

#pragma once

class CSkinnedDialog; // forward declaration

class CSkinnedWndCompanionObj
{
public:

	// CONSTRUCTION

	CSkinnedWndCompanionObj( CSkinnedDialog* pdlgSkinnedDialog, UINT nIDSkin, COLORREF clrSkinBkgrd );
	~CSkinnedWndCompanionObj ();

	// INITIALIZATION

	BOOL Initialize();

	// METHODS

	void GetClientRectInScreenCoords( CRect& rctClientRect );

	void GetMinTrackSize( long& lX, long& lY );
	void SetMinTrackSize( long lX, long lY );

	// HANDLERS

	afx_msg BOOL OnEraseBkgnd( CDC* pDC );
	afx_msg void OnPaint ();
	afx_msg void OnNcPaint();
	afx_msg void OnGetMinMaxInfo( MINMAXINFO FAR* lpMMI );
	afx_msg void OnSize( UINT nType, int cx, int cy );
	afx_msg UINT OnNcHitTest( CPoint point );
	afx_msg BOOL OnNcActivate( BOOL bActive );
	afx_msg void OnNcLButtonDown( UINT nHitTest, CPoint point );
	afx_msg void OnNcLButtonUp( UINT nHitTest, CPoint point );
	afx_msg void OnNcRButtonDown( UINT nHitTest, CPoint point );
	afx_msg void OnNcRButtonUp( UINT nHitTest, CPoint point );
	afx_msg void OnLButtonDown( UINT nFlags, CPoint point );
	afx_msg void OnLButtonUp( UINT nFlags, CPoint point );
	afx_msg void OnRButtonDown( UINT nFlags, CPoint point );
	afx_msg void OnRButtonUp( UINT nFlags, CPoint point );
    afx_msg void OnMouseMove( UINT nFlags, CPoint point );
	afx_msg void OnNcMouseMove( UINT nHitTest, CPoint point );
	afx_msg BOOL OnSetCursor( CWnd* pWnd, UINT nHitTest, UINT message );
	afx_msg void OnInitMenu( CMenu* pMenu );
	afx_msg void OnLButtonDblClk( UINT nFlags, CPoint point );
	afx_msg void OnNcLButtonDblClk( UINT nHitTest, CPoint point );
	afx_msg void OnCaptureChanged( CWnd* pWnd );
	afx_msg void OnTimer( UINT_PTR nIDEvent );
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);

protected:

	// CONSTRUCTION PARAMS

	CSkinnedDialog*	m_pdlgSkinnedDialog;
	UINT			m_nIDSkin;
	COLORREF		m_clrSkinBkgrd;

	// CLASS INSTANCE DATA

		// window state

		long		m_lMinTrackSizeX; // initialized to -1, set and used only when SetMinTrackSize is called
		long		m_lMinTrackSizeY; // initialized to -1, set and used only when SetMinTrackSize is called

		enum		eCaptionState
		{
			ECS_Normal = 0,
			ECS_Red,
			ECS_Gold
		};
		eCaptionState		m_ecsCaptionState;

		enum		eFirstButtonState
		{
			EFBS_NormalMinimize = 0,
			EFBS_RedMinimize,
			EFBS_GrayMinimize,
			EFBS_GoldMinimize
		};
		eFirstButtonState	m_efbsFirstButtonState;

		enum		eSecondButtonState
		{
			ESBS_NormalMaximize = 0,
			ESBS_RedMaximize,
			ESBS_GrayMaximize,
			ESBS_GoldMaximize,
			ESBS_NormalRestore,
			ESBS_RedRestore,
			ESBS_GoldRestore
		};
		eSecondButtonState	m_esbsSecondButtonState;

		enum		eThirdButtonState
		{
			ETBS_NormalClose = 0,
			ETBS_RedClose,
			ETBS_GoldClose
		};
		eThirdButtonState	m_etbsThirdButtonState;

		typedef struct sUiElementsPos
		{
			RECT		rctCaption;
			RECT		rctFirstButton;
			RECT		rctSecondButton;
			RECT		rctThirdButton;
			RECT		rctBorder[ 4 ];
			RECT		rctClientArea;

		} sUiElementsPos;
		sUiElementsPos		m_uiepUiElementsPos;

		// interaction

		enum		eUserOperation
		{
			EUO_None = 0,
			EUO_MinimizeBoxClicking,
			EUO_MaximizeBoxClicking,
			EUO_RestoreBoxClicking,
			EUO_CloseBoxClicking,
			EUO_RightClickOnCaption,
			EUO_Move,
			EUO_SizeNorth,
			EUO_SizeSouth,
			EUO_SizeWest,
			EUO_SizeEast,
			EUO_SizeNorthWest,
			EUO_SizeNorthEast,
			EUO_SizeSouthWest,
			EUO_SizeSouthEast
		};
		eUserOperation		m_euoUserOperation;
		POINT				m_ptUserOpCursorStart;
		RECT				m_rctUserOpRectStart;

		// drawing

		CBitmap			m_bmpSkin;
		CDC				m_dcMem;
		HBRUSH			m_hbBackgBrush;

	// PROTECTED METHODS

		// interaction

		BOOL LButtonDown ( void );
		BOOL LButtonUp ( void );
		BOOL RButtonDown ( void );
		BOOL RButtonUp ( void );
		BOOL MouseMove ( void );
		BOOL LButtonDblClk ( void );

		// misc

		void DrawSkinnedWndElement( CDC* pCdc, DWORD dwElementOrdinal, int x, int y, int w = 0, int h = 0 );
		void SetWindowRgn ( void );
		void RedrawWindow ( BOOL bRedrawBody = TRUE, BOOL bRedrawOnlyCaption = FALSE );
		void CalcWndElemsPos ( void );
		UINT HitTest ( CPoint point, BOOL* pbIsOnCloseBtn = NULL );
		BOOL UpdateUiElemsState ( BOOL bCalledFromTimerHandler = FALSE, BOOL bRedrawHowever = FALSE );
		void ReleaseSkinWndCapture ( BOOL bCaptureChanged );
		void StartUserOperation( eUserOperation euoUserOperation );

};
